@@ -90,7 +90,41 @@ function(x, ...)
 ### * ls_fit_ultrametric
 
 ls_fit_ultrametric <-
-function(x, control = list())
+function(x, method = c("SUMT", "IP", "IR"), weights = 1,
+         control = list())
+{
+    if(inherits(x, "cl_ultrametric"))
+        return(x)
+
+    ## Handle weights.
+    ## This is somewhat tricky ...
+    if(is.matrix(weights)) {
+        weights <- as.dist(weights)
+        if(length(weights) != length(x))
+            stop("Arguments 'weights' must be compatible with 'x'.")
+    }
+    else
+        weights <- rep(weights, length = length(x))
+    if(any(weights < 0))
+        stop("Argument 'weights' has negative elements.")
+    if(!any(weights > 0))
+        stop("Argument 'weights' has no positive elements.")
+
+    method <- match.arg(method)
+    switch(method,
+           SUMT = .ls_fit_ultrametric_by_SUMT(x, weights, control),
+           IP = {
+               .ls_fit_ultrametric_by_iterative_projection(x, weights,
+                                                           control)
+           },
+           IR = {
+               .ls_fit_ultrametric_by_iterative_reduction(x, weights,
+                                                          control)
+           })
+}
+           
+.ls_fit_ultrametric_by_SUMT <-
+function(x, weights = 1, control = list())
 {
     ## Fit an ultrametric to a dissimilarity by minimizing euclidean
     ## dissimilarity subject to the ultrametric constraint, using the
@@ -99,6 +133,13 @@ function(x, control = list())
     ## "exactly" rather than approximately.  We (currently?) do that via
     ## rounding ...
 
+    ## <NOTE>
+    ## This fits and hence returns an ultrametric, *not* the hierarchy
+    ## corresponding to the ultrametric.
+    ## </NOTE>
+
+    w <- weights / sum(weights)
+
     ## Control parameters.
     eps <- control$eps
     ## <TODO>
@@ -130,14 +171,23 @@ function(x, control = list())
     ## For the time being, use a simple minimizer.
 
     n <- attr(x, "Size")
-    if(n <= 1) return(x)
+    if(n <= 2) return(as.cl_ultrametric(x))
     labels <- attr(x, "Labels")
+
+    ## Handle missing values in x along the lines of de Soete (1984):
+    ## set the corresponding weights to 0, and impute by the weighted
+    ## mean.
+    ind <- which(is.na(x))
+    if(any(ind)) {
+        w[ind] <- 0
+        x[ind] <- weighted.mean(x, w, na.rm = TRUE)
+    }
     
     ## We follow de Soete's notation, and use the veclh's (vector of
     ## lower half, in S the same as x[lower.tri(x)]) of the respective
     ## proximity objects.
 
-    L <- function(d) sum((x - d) ^ 2)
+    L <- function(d) sum(w * (x - d) ^ 2)
     P <- function(d) {
         ## Smooth penalty function measuring the extent of violation of
         ## the ultrametricity constraint.
@@ -146,14 +196,14 @@ function(x, control = list())
     grad_P <- function(d) {
         d <- .symmetric_matrix_from_veclh(d, n)
         n <- nrow(d)
-        gr <- matrix(.C("clue_ultrametric_penalty_gradient",
-                        d, n, gr = double(n * n),
+        gr <- matrix(.C("deviation_from_ultrametricity_gradient",
+                        as.double(d), n, gr = double(n * n),
                         PACKAGE = "clue")$gr, n, n)
         gr[row(gr) > col(gr)]
     }
     Phi <- function(rho, d) L(d) + rho * P(d)
     grad_Phi <- function(rho, d) {
-        2 * (d - x) + rho * grad_P(d)
+        2 * w * (d - x) + rho * grad_P(d)
     }
 
     make_Phi <- if(method == "nlm") {
@@ -220,12 +270,189 @@ function(x, control = list())
     }
 
     ## Round to enforce ultrametricity, and hope for the best ...
-    mnum <- .non_ultrametricity(d, max = TRUE)
     ## Alternatively, we could try running one more optimization step
     ## with just the penalty function.
-    cl_ultrametric(as.cl_ultrametric(round(d, floor(abs(log10(mnum))))),
-                   size = n, labels = labels)
+    .cl_ultrametric_from_ultrametric_approximation(d, size = n,
+                                                   labels = labels)
+}
 
+### * .ls_fit_ultrametric_by_iterative_projection
+
+## <NOTE>
+## Functions
+##   .ls_fit_ultrametric_by_iterative_projection()
+##   .ls_fit_ultrametric_by_iterative_reduction()
+## are really identical apart from the name of the C routine they call.
+## (But will this necessarily always be the case in the future?)
+## Merge maybe ...
+## </NOTE>
+
+.ls_fit_ultrametric_by_iterative_projection <-
+function(x, weights = 1, control = list())
+{
+    if(any(diff(weights)))
+        warning("Non-identical weights currently not supported.")
+
+    if(attr(x, "Size") <= 2) return(as.cl_ultrametric(x))
+
+    labels <- attr(x, "Labels")
+    x <- as.matrix(x)
+    n <- nrow(x)
+
+    ## Control parameters:
+    ## maxiter,
+    maxiter <- control$maxiter
+    if(is.null(maxiter))
+        maxiter <- 10000
+    ## order,
+    order <- control$order
+    if(is.null(order))
+        order <- 1 : n
+    else {
+        if(!all(sort(order) == 1 : n))
+            stop("Given order is not a valid permutation.")
+    }
+    ## tol,
+    tol <- control$tol
+    if(is.null(tol))
+        tol <- 1e-8
+    ## verbose.
+    verbose <- control$verbose
+    if(is.null(verbose))
+        verbose <- getOption("verbose")
+    
+    out <- .C("ls_fit_ultrametric_by_iterative_projection",
+              as.double(as.matrix(x)),
+              as.integer(n),
+              as.integer(order - 1),
+              as.integer(maxiter),
+              iter = integer(1),
+              as.double(tol),
+              as.logical(verbose))
+    d <- as.dist(matrix(out[[1]], n))
+    .cl_ultrametric_from_ultrametric_approximation(d, n, labels)
+}
+
+### * .ls_fit_ultrametric_by_iterative_reduction
+
+.ls_fit_ultrametric_by_iterative_reduction <-
+function(x, weights = 1, control = list())
+{
+    if(any(diff(weights)))
+        warning("Non-identical weights currently not supported.")
+
+    if(attr(x, "Size") <= 2) return(as.cl_ultrametric(x))
+
+    labels <- attr(x, "Labels")
+    x <- as.matrix(x)
+    n <- nrow(x)
+
+    ## Control parameters:
+    ## maxiter,
+    maxiter <- control$maxiter
+    if(is.null(maxiter))
+        maxiter <- 10000
+    ## order,
+    order <- control$order
+    if(is.null(order))
+        order <- 1 : n
+    else {
+        if(!all(sort(order) == 1 : n))
+            stop("Given order is not a valid permutation.")
+    }
+    ## tol,
+    tol <- control$tol
+    if(is.null(tol))
+        tol <- 1e-8
+    ## verbose.
+    verbose <- control$verbose
+    if(is.null(verbose))
+        verbose <- getOption("verbose")
+
+    out <- .C("ls_fit_ultrametric_by_iterative_reduction",
+              as.double(x),
+              as.integer(n),
+              as.integer(order - 1),
+              as.integer(maxiter),
+              iter = integer(1),
+              as.double(tol),
+              as.logical(verbose),
+              PACKAGE = "clue")
+    d <- as.dist(matrix(out[[1]], n))
+    ## <NOTE>
+    ## If we want to add an attribute with the convergence info, we need
+    ## to do this after using as.cl_ultrametric().
+    ## </NOTE>
+    .cl_ultrametric_from_ultrametric_approximation(d, n, labels)
+}
+
+### * Ultrametric Target Fitters.
+
+### ** ls_fit_ultrametric_target
+
+ls_fit_ultrametric_target <-
+function(x, y, weights = 1)
+{
+    if(identical(weights, 1))           # Default.
+        fitter <- function(x, w) mean(x)
+    else
+        fitter <- function(x, w) weighted.mean(x, w)
+    .fit_ultrametric_target(x, y, weights, fitter)
+}
+
+### ** l1_fit_ultrametric_target
+
+l1_fit_ultrametric_target <-
+function(x, y, weights = 1)
+{
+    if(identical(weights, 1))           # Default.
+        fitter <- function(x, w) median(x)
+    else
+        fitter <- function(x, w) weighted_median(x, w)
+    .fit_ultrametric_target(x, y, weights, fitter)
+}
+
+### ** .fit_ultrametric_target    
+
+.fit_ultrametric_target <-
+function(x, y, w, fitter)
+{
+    w <- .handle_weights_for_ultrametric_target_fitters(w, x)
+    x <- as.matrix(x)
+    y <- as.hclust(y)
+    n <- length(y$order)
+    ilist <- vector("list", n)
+    out <- matrix(0, n, n)
+    for(i in 1 : (n - 1)) {
+        inds <- y$merge[i, ]
+        ids1 <- if(inds[1] < 0) -inds[1] else ilist[[inds[1]]]
+        ids2 <- if(inds[2] < 0) -inds[2] else ilist[[inds[2]]]
+        ilist[[i]] <- c(ids1, ids2)
+        out[ids1, ids2] <- fitter(x[ids1, ids2], w[ids1, ids2])
+    }
+    rownames(out) <- y$labels
+    as.cl_ultrametric(out + t(out))
+}
+
+### ** .handle_weights_for_ultrametric_target_fitters
+
+.handle_weights_for_ultrametric_target_fitters <-
+function(weights, x)
+{
+    ## Handle weights for the ultrametric target fitters.
+    ## This is somewhat tricky ...
+    if(is.matrix(weights)) {
+        if(any(dim(weights) != attr(x, "Size")))
+            stop("Arguments 'weights' must be compatible with 'x'.")
+    }
+    else
+        weights <- as.matrix(.dist_from_vector(rep(weights, length =
+                                                   length(x)))) 
+    if(any(weights < 0))
+        stop("Argument 'weights' has negative elements.")
+    if(!any(weights > 0))
+        stop("Argument 'weights' has no positive elements.")
+    weights
 }
 
 ### * .dist_from_vector
@@ -265,51 +492,24 @@ function(x, max = FALSE)
 {
     if(!is.matrix(x))
         x <- .symmetric_matrix_from_veclh(x)
-    if(max) {
-        ## Maybe make this faster in C as well ...
-        max(sapply(seq(length = nrow(x)),
-                   function(k) {
-                       z <- x[, k]
-                       max(x - outer(z, z, pmax))
-                   }))
-    }
-    else
-        .C("clue_ultrametric_penalty_function",
-           x, nrow(x), fn = double(1),
-           PACKAGE = "clue")$fn
+    .C("deviation_from_ultrametricity",
+       as.double(x),
+       nrow(x),
+       fn = double(1),
+       as.logical(max),
+       PACKAGE = "clue")$fn
 }
 
-### .get_classes_in_hierarchy
+### .cl_ultrametric_from_ultrametric_approximation
 
-## <FIXME>
-## We should really have a class for representing n-trees.
-## In principle, these could be considered as hierarchies in the clue
-## sense, because there is always one "minimal" standardized ultrametric
-## representing them.
-## But let's think about this some more, and perhaps just add class
-## "n_tree" to what is returned by .get_classes_in_hierarchy().
-## Then we could have .cl_ultrametric_from_classes() be
-## as.cl_ultrametric.n_tree() ...
-## Of course, we could also turn .get_classes_in_hierarchy() into (a
-## method of) as.n_tree().
-## (Still, not entirely sure about this ...)
-## </FIXME>
-
-.get_classes_in_hierarchy <-
-function(x)
+.cl_ultrametric_from_ultrametric_approximation <-
+function(x, size = NULL, labels = NULL)
 {
-    x <- as.hclust(x)
-    n <- n_of_objects(x)
-    labels <- seq(length = n)
-    groups <- cutree(x, labels)
-    ## Give a list with the (unique) sets of numbers of the objects.
-    out <- unique(unlist(sapply(split(groups, col(groups)),
-                                function(k) split(labels, k)),
-                         recursive = FALSE,
-                         use.names = FALSE))
-    ## Preserve labels if possible, and re-order according to
-    ## cardinality.
-    structure(out[order(sapply(out, length))], labels = x$labels)
+    ## Turn x into an ultrametric after possibly rounding to
+    ## non-ultrametric significance.
+    mnum <- .non_ultrametricity(x, max = TRUE)
+    cl_ultrametric(as.cl_ultrametric(round(x, floor(abs(log10(mnum))))), 
+                   size = size, labels = labels)
 }
 
 ### .cl_ultrametric_from_classes
